/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: ipi_2s.cxx,v $
 *
 *  $Revision: 1.11 $
 *
 *  last change: $Author: obo $ $Date: 2006/09/16 16:26:25 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_autodoc.hxx"


#include <precomp.h>
#include "ipi_2s.hxx"


// NOT FULLY DEFINED SERVICES
#include <algorithm>
#include <string.h>
#include <ary/qualiname.hxx>
#include <ary/idl/i_enum.hxx>
#include <ary/idl/i_exception.hxx>
#include <ary/idl/i_function.hxx>
#include <ary/idl/i_gate.hxx>
#include <ary/idl/i_interface.hxx>
#include <ary/idl/ik_interface.hxx>
#include <ary/idl/i_module.hxx>
#include <ary/idl/i_property.hxx>
#include <ary/idl/i_service.hxx>
#include <ary/idl/i_singleton.hxx>
#include <ary/idl/i_siservice.hxx>
#include <ary/idl/i_sisingleton.hxx>
#include <ary/idl/i_struct.hxx>
#include <ary/idl/i_structelem.hxx>
#include <ary/idl/i_typedef.hxx>
#include <ary/idl/ip_ce.hxx>
#include <ary/idl/ip_type.hxx>
#include <store/st_access.hxx>
#include <namesort.hxx>
#include <nametreenode.hxx>
#include "i_nnfinder.hxx"
#include "is_ce.hxx"
#include "is_type.hxx"
#include "it_ce.hxx"
#include "it_explicit.hxx"
#include "it_sequence.hxx"
#include "it_xnameroom.hxx"



namespace ary
{
namespace idl
{

inline Service *
SecondariesPilot_Inst::lhf_SearchService( Type_id i_nType )
{
    return store::search_access( my_CeStorage(),
                                 lhf_Search_CeFromTypeId(i_nType),
                                 T2T<Service>() );
}

inline Interface *
SecondariesPilot_Inst::lhf_SearchInterface( Type_id i_nType )
{
    return store::search_access( my_CeStorage(),
                                 lhf_Search_CeFromTypeId(i_nType),
                                 T2T<Interface>() );
}

inline Struct *
SecondariesPilot_Inst::lhf_SearchStruct( Type_id i_nType )
{
    return store::search_access( my_CeStorage(),
                                 lhf_Search_CeFromTypeId(i_nType),
                                 T2T<Struct>() );
}

inline Exception *
SecondariesPilot_Inst::lhf_SearchException( Type_id i_nType )
{
    return store::search_access( my_CeStorage(),
                                 lhf_Search_CeFromTypeId(i_nType),
                                 T2T<Exception>() );
}

inline const Ce_Storage &
SecondariesPilot_Inst::my_CeStorage() const
    { return *pCeStorage; }

inline const Type_Storage &
SecondariesPilot_Inst::my_TypeStorage() const
    { return *pTypeStorage; }

inline Ce_Storage &
SecondariesPilot_Inst::my_CeStorage()
    { return *pCeStorage; }

inline Type_Storage &
SecondariesPilot_Inst::my_TypeStorage()
    { return *pTypeStorage; }

inline void
SecondariesPilot_Inst::insert_into2sList( CodeEntity &        o_out,
                                          int                 i_listIndex,
                                          Ce_id               i_nCe )
    { o_out.Secondaries().Access_List(i_listIndex).push_back(i_nCe); }


SecondariesPilot_Inst::SecondariesPilot_Inst( Ce_Storage &      i_rCes,
                                              Type_Storage &    i_rTypes )
    :   pCeStorage(&i_rCes),
        pTypeStorage(&i_rTypes)
{
}

SecondariesPilot_Inst::~SecondariesPilot_Inst()
{
}

void
SecondariesPilot_Inst::CheckAllInterfaceBases( Gate & i_rGate )
{
    Module &
        rGlobalNamespace = i_rGate.Ces().GlobalNamespace();
    QualifiedName
        aXInterface("::com::sun::star::uno::XInterface","::");

    const Type &
        rType = i_rGate.Types().CheckIn_Type(
                                        aXInterface,
                                        0,
                                        rGlobalNamespace.CeId(),
                                        Type_id( 0 ) );
    Type_id
        nTypeXInterface = rType.TypeId();
    const ExplicitType &
        rExplType = cref_cast(rType, T2T<ExplicitType>());
    Ce_id
        nCeXInterface = lhf_Search_CeForType(rExplType);

    for ( store::RootFilterIterator<Ce_Storage::unit, Interface>
            it = my_CeStorage().Container().Begin();
          it;
          ++it )
    {
        Interface &
                rInterface = ary_cast<Interface>((*it).Entity());

        if ( NOT rInterface.HasBase()               // According to UNO IDL syntax, an interface without base has com::sun::star::uno::XInterface as base.
             AND rInterface.CeId() != nCeXInterface )   // XInterface must not be base of itself.
        {
            rInterface.Add_Base(nTypeXInterface, 0);
        }
    }   // end for
}

void
SecondariesPilot_Inst::do_Connect_Types2Ces()
{
    for ( store::RootFilterIterator<Type_Storage::unit, ExplicitType>
            it = my_TypeStorage().Container().Begin();
          it;
          ++it )
    {
        ExplicitType &
            rType = ref_cast((*it).Entity(), T2T<ExplicitType>());
        Ce_id
            nRelatedCe = lhf_Search_CeForType(rType);
        if (nRelatedCe.IsValid())
        {
            Ce_Type * pNew = new Ce_Type(nRelatedCe, rType.TemplateType());
            pNew->Set_Id(rType.Id());
            (*it).Set_Entity(*pNew);
        }
    }   // end for
}

void
SecondariesPilot_Inst::do_Gather_CrossReferences()
{
    gather_Synonyms();

    for ( store::RootIterator<Ce_Storage::unit>
            it = my_CeStorage().Container().Begin();
          it;
          ++it )
    {
        (*it).Entity().Visit( static_cast<CeHost& >(*this) );
    }   // end for

    sort_All2s();
}

namespace
{

enum E_LinkMode
{
    link2descr,
    link2ref
};

}


void
SecondariesPilot_Inst::do_Read_Links2DevManual( csv::bstream & i_file )
{
    StreamLock  aLine(300);
    StreamStr & rLine = aLine();


    String      sCurLink;
    String      sCurLinkUI;
    E_LinkMode  eCurMode = link2ref;

	int lineCount = 0;


    while (NOT i_file.eod())
    {
		++lineCount;

        rLine.reset();
        rLine.operator_read_line(i_file);

        if ( *rLine.c_str() >= 'a' )
        {
            assign_CurLink(rLine.begin(), sCurLink, sCurLinkUI, eCurMode == link2descr, lineCount);
        }
        else if ( strncmp(rLine.c_str(),"LINK:",5) == 0 )
        {
            sCurLink = rLine.c_str()+5;
            sCurLinkUI.clear();
        }
        else if ( strncmp(rLine.c_str(),"DESCR:",6) == 0 )
        {
            sCurLinkUI = rLine.c_str()+6;
        }
        else if ( strncmp(rLine.c_str(),"TOPIC:",6) == 0 )
        {
            eCurMode = link2descr;
        }
        else if ( strncmp(rLine.c_str(),"REF:",4) == 0 )
        {
            eCurMode = link2ref;
        }
        else if (*rLine.c_str() >= 32)
        {
            assign_CurLink(rLine.begin(), sCurLink, sCurLinkUI, eCurMode == link2descr, lineCount);
        }
    }   // end while
}


struct OrderCeIdsByName
{
                    OrderCeIdsByName(
                        const Ce_Storage::container &
                                                i_storage )
                                                : rStorage(i_storage),
                                                  aNameComparison() {}

    bool            operator()(
                        Ce_id                   i_ce1,
                        Ce_id                   i_ce2 ) const
    {
        return aNameComparison( rStorage[i_ce1].Entity().LocalName(),
                                rStorage[i_ce2].Entity().LocalName() );
    }

  private:
    const Ce_Storage::container &
                    rStorage;
    CompareCeNames  aNameComparison;
};


const int C_nLowerUpperDiff = 'a'-'A';

void
SecondariesPilot_Inst::inq_Get_AlphabeticalIndex(
                            std::vector<Ce_id> &            o_rResult,
                            alphabetical_index::E_Letter    i_cLetter ) const
{
    // Establishing filter:
    char filter[256];

    int nLetter = i_cLetter;
    memset(filter, 0, 256);
    filter[nLetter] = 1;
    if ( i_cLetter != alphabetical_index::non_alpha )
        filter[nLetter - C_nLowerUpperDiff] = 1;

    // Gather entities which start with i_cLetter:
    o_rResult.reserve(1000);
    typedef store::RootConstIterator<Ce_Storage::unit>     Iter;
    for ( Iter it = my_CeStorage().Container().Begin();
          it;
          ++it )
    {
        if ( filter[ int(*(*it).Entity().LocalName().c_str()) ] == 1 )
            o_rResult.push_back( (*it).Entity().CeId() );
    }

    // Sort:
    OrderCeIdsByName    aIdOrdering(my_CeStorage().Container());
    std::sort( o_rResult.begin(),
               o_rResult.end(),
               aIdOrdering );
}

void
SecondariesPilot_Inst::do_Service( const CodeEntity & i_rData )
{
    const Service &
        rService = ary_cast<Service>(i_rData);

    // Interfaces:
    assignImplementation_toAServicesInterfaces( rService.CeId(),
												rService.CeId(),
												interface_2s_ExportingServices );
    // Services and their interfaces:
    recursive_AssignIncludingService(rService.CeId(), rService);
}

void
SecondariesPilot_Inst::do_Interface( const CodeEntity &  i_rData )
{
    assign_AsDerivedInterface( ary_cast<Interface>(i_rData) );
}

void
SecondariesPilot_Inst::do_Struct( const CodeEntity &  i_rData )
{
    assign_AsDerivedStruct( ary_cast<Struct>(i_rData) );
}

void
SecondariesPilot_Inst::do_Exception( const CodeEntity &  i_rData )
{
    assign_AsDerivedException( ary_cast<Exception>(i_rData) );
}

void
SecondariesPilot_Inst::do_Typedef( const CodeEntity &  )
{
    // KORR
    // Find out what was meant here ???

//    const Typedef &
//        rTypedef = ary_cast<Typedef>(i_rData);
}

void
SecondariesPilot_Inst::do_Singleton( const CodeEntity &  i_rData )
{
    const Singleton &
        rSingleton = ary_cast<Singleton>(i_rData);

    Service *
        pServ = lhf_SearchService(rSingleton.AssociatedService());
    if (pServ != 0)
    {
        insert_into2sUnique( *pServ,
							 service_2s_InstantiatingSingletons,
							 rSingleton.CeId() );
    }

    // Interfaces:
    assignImplementation_toAServicesInterfaces( rSingleton.CeId(),
												lhf_Search_CeFromTypeId(rSingleton.AssociatedService()),
												interface_2s_ExportingSingletons );
}

void
SecondariesPilot_Inst::do_SglIfcService( const CodeEntity &  i_rData )
{
    const SglIfcService &
        rSglIfcService = ary_cast<SglIfcService>(i_rData);

    assignImplementation_toAServicesInterfaces( rSglIfcService.CeId(),
												rSglIfcService.CeId(),
												interface_2s_ExportingServices );
}

void
SecondariesPilot_Inst::do_SglIfcSingleton( const CodeEntity &  i_rData )
{
    const SglIfcSingleton &
        rSglIfcSingleton = ary_cast<SglIfcSingleton>(i_rData);

    Type_id nBase = rSglIfcSingleton.BaseInterface();
    recursive_AssignImplementation_toExportedInterface( rSglIfcSingleton.CeId(),
														nBase,
														interface_2s_ExportingSingletons );
}

void
SecondariesPilot_Inst::do_Function( const CodeEntity & i_rData )
{
    const Function &
        rFunction = ary_cast<Function>(i_rData);

    recursive_AssignFunction_toCeAsReturn(rFunction.CeId(), rFunction.ReturnType());

    for ( Function::ParamList::const_iterator itp = rFunction.Parameters().begin();
          itp != rFunction.Parameters().end();
          ++itp )
    {
        recursive_AssignFunction_toCeAsParameter(rFunction.CeId(), (*itp).Type());
    }   // end for (itp)

    for ( Function::ExceptionList::const_iterator itx = rFunction.Exceptions().begin();
          itx != rFunction.Exceptions().end();
          ++itx )
    {
        Exception *
            pX = lhf_SearchException(*itx);
        if (pX != 0)
        {
            insert_into2sUnique(*pX, exception_2s_RaisingFunctions, rFunction.CeId());
        }
    }   // end for (itx)
}

void
SecondariesPilot_Inst::do_StructElement( const CodeEntity & i_rData )
{
    const StructElement &
        rStructElement = ary_cast<StructElement>(i_rData);

    recursive_AssignStructElement_toCeAsDataType(rStructElement.CeId(), rStructElement.Type());
}

void
SecondariesPilot_Inst::do_Property( const CodeEntity & i_rData )
{
    const Property &
        rProperty = ary_cast<Property>(i_rData);

    recursive_AssignStructElement_toCeAsDataType(rProperty.CeId(), rProperty.Type());
}

Ce_id
SecondariesPilot_Inst::lhf_Search_CeForType( const ExplicitType & i_rType ) const
{
    const ExplicitNameRoom &
        rExplicitNameRoom = store::find( my_TypeStorage(),
                                         i_rType.NameRoom(),
                                         T2T<ExplicitNameRoom>() );
    Find_ModuleNode
        rNodeFinder( my_CeStorage(),
                     rExplicitNameRoom.NameChain_Begin(),
                     rExplicitNameRoom.NameChain_End(),
                     i_rType.Name() );

    if ( rExplicitNameRoom.IsAbsolute() )
    {
        const Module &
            rGlobalNamespace = store::find( my_CeStorage(),
                                            Ce_id(predefined::ce_GlobalNamespace),
                                            T2T<Module>() );
        return Search_SubTree(  rGlobalNamespace,
                                rNodeFinder );
    }
    else
    {
        const Module &
            rStartModule = store::find( my_CeStorage(),
                                        i_rType.ModuleOfOccurrence(),
                                        T2T<Module>() );

        Ce_id ret = Search_SubTree_UpTillRoot( rStartModule,
											   rNodeFinder );
		return ret;
    }   // endif (rExplicitNameRoom.IsAbsolute()) else
}

Ce_id
SecondariesPilot_Inst::lhf_Search_CeFromTypeId( Type_id i_nType ) const
{
    const Ce_Type *
        pType = store::search( my_TypeStorage(), i_nType, T2T<Ce_Type>() );
    return pType != 0
                ?   pType->RelatedCe()
                :   Ce_id_Null();
}

void
SecondariesPilot_Inst::assign_CurLink( char *              i_text,
                                       const String &      i_link,
                                       const String &      i_linkUI,
                                       bool                i_isDescr,
                                       int                 i_lineCount )
{
    const ary::idl::Module *
        pModule = &store::find(my_CeStorage(), Ce_id(predefined::ce_GlobalNamespace), T2T<Module>());

    char * pPastNext = 0;
    char * pNext = i_text;
    for ( ;
          (pPastNext = strstr(pNext,".")) != 0;
          pNext = pPastNext + 1 )
    {
        String sNext(pNext, pPastNext-pNext);
        Ce_id nModule = pModule->Search_Name(sNext);
        if (nModule.IsValid())
        {
            pModule = store::search( my_CeStorage(),
                                     nModule,
                                     T2T<Module>() );
        }
        else
        {
            pModule = 0;
        }

        if (pModule == 0)
        {
            Cerr() << "Warning: Invalid line nr. "
                   << i_lineCount
                   << " in DevelopersGuide reference file:\n"
                   << reinterpret_cast< const char* >(i_text)
                   << "\n"
                   << Endl();
            return;
        }
    }   // end for

    pPastNext = strchr(pNext,':');
    bool bMember = pPastNext != 0;
    String sCe( pNext, (bMember ? csv::str::size(pPastNext-pNext) : csv::str::maxsize) );

//  KORR_FUTURE
//  String sMember(bMember ? pPastNext+1, "");

    Ce_id nCe = pModule->Search_Name(sCe);
    if (NOT nCe.IsValid())
    {
        Cerr() << "Warning: Invalid line nr. "
               << i_lineCount
               << " in DevelopersGuide reference file:\n"
               << reinterpret_cast< const char* >(i_text)
               << "\n"
               << Endl();
        return;
    }

    CodeEntity &
        rCe = my_CeStorage()[nCe].Entity();
    if (NOT bMember)
    {
        if (i_isDescr)
            rCe.Secondaries().Add_Link2DescriptionInManual(i_link, i_linkUI);
        else
            rCe.Secondaries().Add_Link2RefInManual(i_link, i_linkUI);
        return;
    }
    else
    {
    // KORR_FUTURE
    //   Provisorial just doing nothing (or may be
    //   adding a link at main Ces lists).
//    if (i_isDescr)
//        rCe.Secondaries().Add_Link2DescriptionInManual(i_link);
//    else
//        rCe.Secondaries().Add_Link2RefInManual(i_link);
    }
}

void
SecondariesPilot_Inst::gather_Synonyms()
{
    for ( store::RootFilterIterator<Ce_Storage::unit, Typedef>
            itTd = my_CeStorage().Container().Begin();
          itTd;
          ++itTd )
    {
        const Typedef &
                rTypedef = ary_cast<Typedef>((*itTd).Entity());

        recursive_AssignAsSynonym(rTypedef.CeId(), rTypedef);
    }   // end for (itTd)
}

void
SecondariesPilot_Inst::recursive_AssignAsSynonym( Ce_id               i_synonymousTypedefsId,
                                                  const Typedef &     i_TypedefToCheck )
{
    Ce_id
        nCe = lhf_Search_CeFromTypeId(i_TypedefToCheck.DefiningType());
    if (NOT nCe.IsValid())
        return;
    CodeEntity &
        rCe = my_CeStorage()[nCe].Entity();

    switch (rCe.ClassId())  // KORR_FUTURE: make this faster, remove switch.
    {
        case Interface::class_id:
                    insert_into2sList( rCe,
									   interface_2s_SynonymTypedefs,
									   i_synonymousTypedefsId );
                    break;
        case Struct::class_id:
                    insert_into2sList( rCe,
									   struct_2s_SynonymTypedefs,
									   i_synonymousTypedefsId );
                    break;
        case Enum::class_id:
                    insert_into2sList( rCe,
									   enum_2s_SynonymTypedefs,
									   i_synonymousTypedefsId );
                    break;
        case Typedef::class_id:
                    insert_into2sList( rCe,
									   typedef_2s_SynonymTypedefs,
									   i_synonymousTypedefsId );
                    recursive_AssignAsSynonym( i_synonymousTypedefsId,
                                               static_cast< Typedef& >(rCe) );
                    break;
                // default: do nothing.
    }
}

void
SecondariesPilot_Inst::recursive_AssignIncludingService( Ce_id               i_includingServicesId,
                                                         const Service &     i_ServiceToCheckItsIncludes )
{
    Dyn_StdConstIterator<CommentedRelation>
        pIncludedServices;
    i_ServiceToCheckItsIncludes.Get_IncludedServices(pIncludedServices);

    for ( StdConstIterator<CommentedRelation> &
                itServ = *pIncludedServices;
          itServ;
          ++itServ )
    {
        Service *
            pServ = lhf_SearchService((*itServ).Type());
        if (pServ != 0)
        {
            insert_into2sUnique( *pServ,
								 service_2s_IncludingServices,
								 i_includingServicesId
							   );
            recursive_AssignIncludingService(i_includingServicesId, *pServ);

        }   // end if

        assignImplementation_toAServicesInterfaces( i_includingServicesId,
													lhf_Search_CeFromTypeId( (*itServ).Type() ),
													interface_2s_ExportingServices );
    }   // end for
}

void
SecondariesPilot_Inst::assign_AsDerivedInterface( const Interface & i_rDerived )
{
    ary::Dyn_StdConstIterator<ary::idl::CommentedRelation>
        pHelp;
    ary::idl::ifc_interface::attr::Get_Bases(pHelp, i_rDerived);

    for ( ary::StdConstIterator<ary::idl::CommentedRelation> & it = *pHelp;
          it.operator bool();
          ++it )
    {
        Interface *
            pIfc = lhf_SearchInterface( (*it).Type() );
        if (pIfc == 0)
            continue;

        insert_into2sList( *pIfc,
                           interface_2s_Derivations,
						   i_rDerived.CeId() );
    }   // end for
}

void
SecondariesPilot_Inst::assign_AsDerivedStruct( const Struct &   i_rDerived )
{
    Type_id
        nBase = i_rDerived.Base();
    if (nBase.IsValid())
    {
        Struct *
            pParent = lhf_SearchStruct(nBase);
        if (pParent != 0)
        {
            insert_into2sList( *pParent,
							   struct_2s_Derivations,
							   i_rDerived.CeId() );
        }
    }
}

void
SecondariesPilot_Inst::assign_AsDerivedException( const Exception &   i_rDerived )
{
    Type_id
        nBase = i_rDerived.Base();
    if (nBase.IsValid())
    {
        Exception *
            pParent = lhf_SearchException(nBase);
        if (pParent != 0)
        {
            insert_into2sList( *pParent,
							   exception_2s_Derivations,
							   i_rDerived.CeId() );
        }   // end if
    }   // end if
}

void
SecondariesPilot_Inst::assignImplementation_toAServicesInterfaces(
                                                    Ce_id               i_nImpl,
                                                    Ce_id               i_nService,
                                                    E_2s_of_Interface   i_eList )
{
    Service *
        pService = store::search_access( my_CeStorage(),
										 i_nService,
										 T2T<Service>() );

    SglIfcService *
        pSglIfcService = store::search_access( my_CeStorage(),
											   i_nService,
											   T2T<SglIfcService>() );
    if (pService != 0)
    {
        Dyn_StdConstIterator<CommentedRelation>
            pSupportedInterfaces;
        pService->Get_SupportedInterfaces(pSupportedInterfaces);

        for ( StdConstIterator<CommentedRelation> &
                    itInfc = *pSupportedInterfaces;
              itInfc.operator bool();
			  ++itInfc )
        {
            recursive_AssignImplementation_toExportedInterface( i_nImpl,
																(*itInfc).Type(),
																i_eList );
        }   // end for
    }
    else if (pSglIfcService != 0)
    {
        Type_id nBase = pSglIfcService->BaseInterface();
        recursive_AssignImplementation_toExportedInterface( i_nImpl,
															nBase,
															i_eList );
    }   // end if
}

void
SecondariesPilot_Inst::recursive_AssignImplementation_toExportedInterface(
                                                    Ce_id               i_nService,
													Type_id             i_nExportedInterface,
													E_2s_of_Interface   i_eList )
{
    Interface *
        pIfc = lhf_SearchInterface(i_nExportedInterface);
    if (pIfc == 0)
        return;

    insert_into2sUnique( *pIfc,
						 i_eList,
						 i_nService );
    Dyn_StdConstIterator<CommentedRelation>
        pBases;
    ary::idl::ifc_interface::attr::Get_Bases(pBases, *pIfc);
    for ( StdConstIterator<CommentedRelation> & it = *pBases;
          it.operator bool();
          ++it )
    {
        recursive_AssignImplementation_toExportedInterface(i_nService, (*it).Type(), i_eList);
    }
}

void
SecondariesPilot_Inst::recursive_AssignFunction_toCeAsReturn( Ce_id         i_nFunction,
                                                              Type_id       i_nReturnType )
{
    Ce_id
        nCe = lhf_Search_CeFromTypeId(i_nReturnType);
    if (NOT nCe.IsValid())
        return;

    CodeEntity &
        rCe = my_CeStorage()[nCe].Entity();
    switch (rCe.ClassId())  // KORR_FUTURE: make this faster, remove switch.
    {
        case Interface::class_id:
                    insert_into2sList( rCe,
									   interface_2s_AsReturns,
									   i_nFunction );
                    break;
        case Struct::class_id:
                    insert_into2sList( rCe,
									   struct_2s_AsReturns,
									   i_nFunction );
                    break;
        case Enum::class_id:
                    insert_into2sList( rCe,
									   enum_2s_AsReturns,
									   i_nFunction );
                    break;
        case Typedef::class_id:
                    insert_into2sList( rCe,
									   typedef_2s_AsReturns,
									   i_nFunction );
                    recursive_AssignFunction_toCeAsReturn( i_nFunction,
                                                           static_cast< Typedef& >(rCe).DefiningType() );
                    break;
        // default: do nothing.
    }
}

void
SecondariesPilot_Inst::recursive_AssignFunction_toCeAsParameter( Ce_id      i_nFunction,
                                                                 Type_id    i_nParameterType )
{
    Ce_id
        nCe = lhf_Search_CeFromTypeId(i_nParameterType);
    if (NOT nCe.IsValid())
        return;

    CodeEntity &
        rCe = my_CeStorage()[nCe].Entity();
    switch (rCe.ClassId())  // KORR_FUTURE: make this faster, remove switch.
    {
        case Interface::class_id:
                    insert_into2sList( rCe,
									   interface_2s_AsParameters,
									   i_nFunction );
                    break;
        case Struct::class_id:
                    insert_into2sList( rCe,
									   struct_2s_AsParameters,
									   i_nFunction );
                    break;
        case Enum::class_id:
                    insert_into2sList( rCe,
									   enum_2s_AsParameters,
									   i_nFunction );
                    break;
        case Typedef::class_id:
                    insert_into2sList( rCe,
									   typedef_2s_AsParameters,
									   i_nFunction );
                    recursive_AssignFunction_toCeAsParameter( i_nFunction,
                                                              static_cast< Typedef& >(rCe).DefiningType() );
                    break;
        // default: do nothing.
    }
}

void
SecondariesPilot_Inst::recursive_AssignStructElement_toCeAsDataType( Ce_id   i_nDataElement,
                                                                     Type_id i_nDataType )
{
    Ce_id
        nCe = lhf_Search_CeFromTypeId(i_nDataType);
    if (NOT nCe.IsValid())
        return;

    CodeEntity &
        rCe = my_CeStorage()[nCe].Entity();
    switch (rCe.ClassId())  // KORR_FUTURE: make this faster, remove switch.
    {
        case Interface::class_id:
                    insert_into2sList( rCe,
									   interface_2s_AsDataTypes,
									   i_nDataElement );
                    break;
        case Struct::class_id:
                    insert_into2sList( rCe,
									   struct_2s_AsDataTypes,
									   i_nDataElement );
                    break;
        case Enum::class_id:
                    insert_into2sList( rCe,
									   enum_2s_AsDataTypes,
									   i_nDataElement );
                    break;
        case Typedef::class_id:
                    insert_into2sList( rCe,
									   typedef_2s_AsDataTypes,
									   i_nDataElement );
                    recursive_AssignFunction_toCeAsParameter( i_nDataElement,
                                                              static_cast< Typedef& >(rCe).DefiningType() );
                    break;
        // default: do nothing.
    }   // end switch
}

void
SecondariesPilot_Inst::insert_into2sUnique( CodeEntity &        o_out,
											int                 i_listIndex,
											Ce_id               i_nCe )
{
    std::vector<Ce_id> &
        rOut = o_out.Secondaries().Access_List(i_listIndex);
    if (std::find(rOut.begin(),rOut.end(),i_nCe) != rOut.end())
        return;
    rOut.push_back(i_nCe);
}

void
SecondariesPilot_Inst::sort_All2s()
{
    OrderCeIdsByName
        aIdOrdering(my_CeStorage().Container());

    for ( store::RootIterator<Ce_Storage::unit>
            it = my_CeStorage().Container().Begin();
          it;
          ++it )
    {
        Ce_2s &
            r2s = (*it).Entity().Secondaries();
        int iCount = r2s.CountXrefLists();
        for (int i = 0; i < iCount; ++i)
        {
            std::sort( r2s.Access_List(i).begin(),
                       r2s.Access_List(i).end(),
                       aIdOrdering );
        }   // end for (i)
    }   // end for (it)
}



}   // namespace idl
}   // namespace ary
