/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: VMeterBase.cxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: obo $ $Date: 2006/09/17 13:32:13 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_chart2.hxx"
#include "VMeterBase.hxx"
#include "ShapeFactory.hxx"
#include "CommonConverters.hxx"
#include "TickmarkHelper.hxx"

// header for define DBG_ASSERT
#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif

//.............................................................................
namespace chart
{
//.............................................................................
using namespace ::com::sun::star;
using namespace ::com::sun::star::chart2;

VMeterBase::VMeterBase( const uno::Reference< XMeter >& xMeter, sal_Int32 nDimensionCount )
            : PlotterBase( nDimensionCount )
            , m_xMeter( xMeter )
{
}

VMeterBase::~VMeterBase()
{
}

void SAL_CALL VMeterBase::setMeterData(
              const ExplicitScaleData& rScale
            , const ExplicitIncrementData& rIncrement )
            throw (uno::RuntimeException)
{
    m_aScale = rScale;
    m_aIncrement = rIncrement;
}

void SAL_CALL VMeterBase::setTransformationSceneToScreen( const drawing::HomogenMatrix& rMatrix )
{
    m_aMatrixScreenToScene = HomogenMatrixToMatrix4D(rMatrix);
    PlotterBase::setTransformationSceneToScreen( rMatrix);
}

TickmarkHelper* VMeterBase::createTickmarkHelper()
{
    TickmarkHelper* pRet=NULL;
    if( 2==m_nDimension )
    {
        sal_Int32 nDimensionIndex = m_xMeter->getRepresentedDimension();
        pRet = new TickmarkHelper( m_aScale, m_aIncrement );
    }
    else
    {
        pRet = new TickmarkHelper_3D( m_aScale, m_aIncrement );
    }
    return pRet;
}

//.............................................................................
} //namespace chart
//.............................................................................
