/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: fubyid.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: obo $ $Date: 2006/09/16 20:09:29 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

// MARKER(update_precomp.py): autogen include statement, do not remove
#include "precompiled_sch.hxx"
#ifndef _SFXDISPATCH_HXX //autogen
#include <sfx2/dispatch.hxx>
#endif
#ifndef _SV_MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif


#ifndef _SFXVIEWFRM_HXX //autogen
#include <sfx2/viewfrm.hxx>
#endif


#include <svx/svxids.hrc>

#include "app.hrc"
#include "schresid.hxx"
#include "strings.hrc"
#include "viewshel.hxx"
#include "schview.hxx"
#include "chwindow.hxx"
#ifndef _CHTMODEL_HXX
#include "chtmodel.hxx"
#endif
#include "attrib.hrc"
#include "attrib.hxx"
#include "unbyid.hxx"
#include "fubyid.hxx"

TYPEINIT1(SchFuById, SchFuPoor);


/*************************************************************************
|*
|* Konstruktor
|*
\************************************************************************/

SchFuById::SchFuById(SchViewShell* pViewSh, SchWindow* pWin,
									 SchView* pView, ChartModel* pDoc,
									 SfxRequest& rReq,long nId)
	: SchFuPoor(pViewSh, pWin, pView, pDoc, rReq)
{
	const SfxItemSet* pArgs = rReq.GetArgs();

	AttrType nAttr;
	switch(nId)
	{
		case CHOBJID_DIAGRAM_FLOOR:
			nAttr=ATTR_DIAGRAM_FLOOR;
			break;
		case CHOBJID_DIAGRAM_STOCKLOSS_GROUP:
			nAttr=ATTR_DIAGRAM_STOCK_LOSS;
			break;
		case CHOBJID_DIAGRAM_STOCKPLUS_GROUP:
			nAttr=ATTR_DIAGRAM_STOCK_PLUS;
			break;
	}
	if (!pArgs)
	{
		SchAttribTabDlg* pDlg =	new SchAttribTabDlg(NULL,
				nAttr,&pDoc->GetAttr(nId),
				pDoc->GetObjectShell(), pDoc, 0, pDoc->ChartStyle());

		if (pDlg->Execute() == RET_OK)
		{
			rReq.Done(*pDlg->GetOutputItemSet());
			pArgs = rReq.GetArgs();
			delete pDlg;
		}
		else
		{
			delete pDlg;
			return;
		}

		pChDoc->CanRebuild (TRUE);
	}
	else pChDoc->CanRebuild (FALSE);

	if( pViewSh )
		pViewSh->GetViewFrame()->GetDispatcher()->Execute(
			SID_TOOL_SELECT, SFX_CALLMODE_SYNCHRON );

	SfxItemSet aOldAttr(pDoc->GetAttr(nId));

	pDoc->ChangeAttr(*pArgs, nId);
	SchUndoById* pUndo = new SchUndoById(*pDoc, aOldAttr, *pArgs,nId);
	switch(nId)
	{
		case CHOBJID_DIAGRAM_FLOOR:
			pUndo->SetComment(String(SchResId(STR_UNDO_DIAGRAM_FLOOR)));
			break;
		case CHOBJID_DIAGRAM_STOCKLOSS_GROUP:
			pUndo->SetComment(String(SchResId(STR_UNDO_DIAGRAM_STOCK_LOSS)));
			break;
		case CHOBJID_DIAGRAM_STOCKPLUS_GROUP:
			pUndo->SetComment(String(SchResId(STR_UNDO_DIAGRAM_STOCK_PLUS)));
			break;
	}
	pViewSh->GetViewFrame()->GetObjectShell()->GetUndoManager()->AddUndoAction(pUndo);
	RemarkObject ();
}


void SchFuById::Activate()
{
}
void SchFuById::Deactivate()
{
}


