/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: SchTransferable.hxx,v $
 *
 *  $Revision: 1.11 $
 *
 *  last change: $Author: kz $ $Date: 2006/12/12 16:03:33 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/
#ifndef _SCH_TRANSFERABLE_HXX_
#define _SCH_TRANSFERABLE_HXX_

#ifndef _TRANSFER_HXX
#include <svtools/transfer.hxx>
#endif

#include <svtools/embedtransfer.hxx>

class SdrModel;
// #i71538# use complete SdrViews
class SdrView; // SdrExchangeView;
class Graphic;
class VirtualDevice;

class SchTransferable :
	public TransferableHelper
{
private:

    SdrModel *                      mpContentModel;
	// #i71538# use complete SdrViews
    // SdrExchangeView  *              mpSourceView;
    SdrView*						mpSourceView;

    TransferableObjectDescriptor    maObjDesc;
    sal_Bool                        mbLateInit;

    SdrModel *                      mpOwnModel;
	// #i71538# use complete SdrViews
    // SdrExchangeView  *              mpOwnView;
    SdrView*						mpOwnView;

    Graphic  *                      mpGraphic;

    bool                            mbDataCreated;
    bool                            mbOwnView;
    bool                            mbOwnDoc;

    void                            CreateData();

protected:

	// implementation of TransferableHelper methods
	virtual void		AddSupportedFormats();
	virtual sal_Bool	GetData( const ::com::sun::star::datatransfer::DataFlavor& rFlavor );
	virtual sal_Bool	WriteObject( SotStorageStreamRef& rxOStm, void* pUserObject, sal_uInt32 nUserObjectId,
									 const ::com::sun::star::datatransfer::DataFlavor& rFlavor );
	virtual void		ObjectReleased();

public:

    /** CTOR for clipboard, drag and drop and selection clipboard
        @param pObjModel the model containing all objects for the clipboard.
                         The transferable becomes the owner of this model.
        @param pSrcView  the view for drag and drop and the selection clipboard
                         The transferable does not care about destruction of this view
        @param rObjDesc  A descriptor object that holds properties like the starting
                         position of a drag action
        @param bLateInit If true, the actual data is created in GetData (on paste),
                         else data is created on construction
     */
    SchTransferable( SdrModel* pObjModel,
		// #i71538# use complete SdrViews
		// SdrExchangeView* pSrcView,
		SdrView* pSrcView,
		const TransferableObjectDescriptor& rObjDesc,
		sal_Bool bLateInit = sal_False );

	virtual ~SchTransferable();

	static const com::sun::star::uno::Sequence< sal_Int8 > & getUnoTunnelId() throw();
	static SchTransferable* getImplementation( com::sun::star::uno::Reference< com::sun::star::uno::XInterface > ) throw();

    // XUnoTunnel ( ::TransferableHelper )
    virtual sal_Int64 SAL_CALL getSomething( const com::sun::star::uno::Sequence< sal_Int8 >& aIdentifier )
		throw( com::sun::star::uno::RuntimeException );

	// #i71538# use complete SdrViews
  	const SdrView* GetView() throw() { return mpSourceView; }
};

#endif	// _SCH_TRANSFERABLE_HXX_
